@@ -6,8 +6,6 @@ Call ``validate(object_name_to_validate)`` to get a dictionary
 with all the detected errors.
 """
 
-from copy import deepcopy
-from typing import Dict, List, Set, Optional
 import ast
 import collections
 import functools
@@ -18,13 +16,14 @@ import pydoc
 import re
 import textwrap
 import tokenize
+from copy import deepcopy
+from typing import Any, Dict, List, Optional, Set
 
 from .docscrape import get_doc_object
 
-
 DIRECTIVES = ["versionadded", "versionchanged", "deprecated"]
 DIRECTIVE_PATTERN = re.compile(
-    r"^\s*\.\. ({})(?!::)".format("|".join(DIRECTIVES)), re.I | re.M
+    r"^\s*\.\. ({})(?!::)".format("|".join(DIRECTIVES)), re.IGNORECASE | re.MULTILINE
 )
 ALLOWED_SECTIONS = [
     "Parameters",
@@ -81,8 +80,7 @@ ERROR_MSGS = {
     "PR06": 'Parameter "{param_name}" type should use "{right_type}" instead '
     'of "{wrong_type}"',
     "PR07": 'Parameter "{param_name}" has no description',
-    "PR08": 'Parameter "{param_name}" description should start with a '
-    "capital letter",
+    "PR08": 'Parameter "{param_name}" description should start with a capital letter',
     "PR09": 'Parameter "{param_name}" description should finish with "."',
     "PR10": 'Parameter "{param_name}" requires a space before the colon '
     "separating the parameter name and type",
@@ -112,14 +110,22 @@ IGNORE_STARTS = (" ", "* ", "- ")
 IGNORE_COMMENT_PATTERN = re.compile("(?:.* numpydoc ignore[=|:] ?)(.+)")
 
 
+def _unwrap(obj):
+    """Iteratively traverse obj.__wrapped__ until first non-wrapped obj found."""
+    while hasattr(obj, "__wrapped__"):
+        obj = obj.__wrapped__
+    return obj
+
+
 # This function gets called once per function/method to be validated.
 # We have to balance memory usage with performance here. It shouldn't be too
 # bad to store these `dict`s (they should be rare), but to be safe let's keep
 # the limit low-ish. This was set by looking at scipy, numpy, matplotlib,
-# and pandas and they had between ~500 and ~1300 .py files as of 2023-08-16.
+# and pandas, and they had between ~500 and ~1300 .py files as of 2023-08-16.
 @functools.lru_cache(maxsize=2000)
 def extract_ignore_validation_comments(
     filepath: Optional[os.PathLike],
+    encoding: str = "utf-8",
 ) -> Dict[int, List[str]]:
     """
     Extract inline comments indicating certain validation checks should be ignored.
@@ -136,7 +142,7 @@ def extract_ignore_validation_comments(
     """
     numpydoc_ignore_comments = {}
     try:
-        file = open(filepath)
+        file = open(filepath, encoding=encoding)
     except (OSError, TypeError):  # can be None, nonexistent, or unreadable
         return numpydoc_ignore_comments
     with file:
@@ -211,7 +217,7 @@ def error(code, **kwargs):
     message : str
         Error message with variables replaced.
     """
-    return (code, ERROR_MSGS[code].format(**kwargs))
+    return code, ERROR_MSGS[code].format(**kwargs)
 
 
 class Validator:
@@ -244,10 +250,10 @@ class Validator:
 
         Examples
         --------
-        >>> Validator._load_obj('datetime.datetime')
+        >>> Validator._load_obj("datetime.datetime")
         <class 'datetime.datetime'>
         """
-        for maxsplit in range(0, name.count(".") + 1):
+        for maxsplit in range(name.count(".") + 1):
             module, *func_parts = name.rsplit(".", maxsplit)
             try:
                 obj = importlib.import_module(module)
@@ -270,9 +276,13 @@ class Validator:
     def is_function_or_method(self):
         return inspect.isfunction(self.obj)
 
+    @property
+    def is_mod(self):
+        return inspect.ismodule(self.obj)
+
     @property
     def is_generator_function(self):
-        return inspect.isgeneratorfunction(self.obj)
+        return inspect.isgeneratorfunction(_unwrap(self.obj))
 
     @property
     def source_file_name(self):
@@ -289,12 +299,18 @@ class Validator:
 
         except TypeError:
             # In some cases the object is something complex like a cython
-            # object that can't be easily introspected. An it's better to
+            # object that can't be easily introspected. And it's better to
             # return the source code file of the object as None, than crash
             pass
         else:
             return fname
 
+    # When calling validate, files are parsed twice
+    @staticmethod
+    @functools.lru_cache(maxsize=4000)
+    def _getsourcelines(obj: Any):
+        return inspect.getsourcelines(obj)
+
     @property
     def source_file_def_line(self):
         """
@@ -302,11 +318,11 @@ class Validator:
         """
         try:
             if isinstance(self.code_obj, property):
-                sourcelines = inspect.getsourcelines(self.code_obj.fget)
+                sourcelines = self._getsourcelines(self.code_obj.fget)
             elif isinstance(self.code_obj, functools.cached_property):
-                sourcelines = inspect.getsourcelines(self.code_obj.func)
+                sourcelines = self._getsourcelines(self.code_obj.func)
             else:
-                sourcelines = inspect.getsourcelines(self.code_obj)
+                sourcelines = self._getsourcelines(self.code_obj)
             # getsourcelines will return the line of the first decorator found for the
             # current function. We have to find the def declaration after that.
             def_line = next(
@@ -319,7 +335,7 @@ class Validator:
             return sourcelines[-1] + def_line
         except (OSError, TypeError):
             # In some cases the object is something complex like a cython
-            # object that can't be easily introspected. An it's better to
+            # object that can't be easily introspected. And it's better to
             # return the line number as None, than crash
             pass
 
@@ -520,9 +536,9 @@ class Validator:
         if tree:
             returns = get_returns_not_on_nested_functions(tree[0])
             return_values = [r.value for r in returns]
-            # Replace NameConstant nodes valued None for None.
+            # Replace Constant nodes valued None for None.
             for i, v in enumerate(return_values):
-                if isinstance(v, ast.NameConstant) and v.value is None:
+                if isinstance(v, ast.Constant) and v.value is None:
                     return_values[i] = None
             return any(return_values)
         else:
@@ -612,7 +628,7 @@ def validate(obj_name, validator_cls=None, **validator_kwargs):
     else:
         doc = validator_cls(obj_name=obj_name, **validator_kwargs)
 
-    # lineno is only 0 if we have a module docstring in the file and we are
+    # lineno is only 0 if we have a module docstring in the file, and we are
     # validating that, so we change to 1 for readability of the output
     ignore_validation_comments = extract_ignore_validation_comments(
         doc.source_file_name
@@ -620,7 +636,29 @@ def validate(obj_name, validator_cls=None, **validator_kwargs):
 
     errs = []
     if not doc.raw_doc:
-        if "GL08" not in ignore_validation_comments:
+        report_GL08: bool = True
+        # Check if the object is a class and has a docstring in the constructor
+        # Also check if code_obj is defined, as undefined for the AstValidator in validate_docstrings.py.
+        if (
+            doc.name.endswith(".__init__")
+            and doc.is_function_or_method
+            and hasattr(doc, "code_obj")
+        ):
+            cls_name = doc.code_obj.__qualname__.split(".")[0]
+            cls = Validator._load_obj(f"{doc.code_obj.__module__}.{cls_name}")
+            # cls = Validator._load_obj(f"{doc.name[:-9]}.{cls_name}") ## Alternative
+            cls_doc = Validator(get_doc_object(cls))
+
+            # Parameter_mismatches, PR01, PR02, PR03 are checked for the class docstring.
+            # If cls_doc has PR01, PR02, PR03 errors, i.e. invalid class docstring,
+            # then we also report missing constructor docstring, GL08.
+            report_GL08 = len(cls_doc.parameter_mismatches) > 0
+
+        # Check if GL08 is to be ignored:
+        if "GL08" in ignore_validation_comments:
+            report_GL08 = False
+        # Add GL08 error?
+        if report_GL08:
             errs.append(error("GL08"))
         return {
             "type": doc.type,
@@ -672,12 +710,18 @@ def validate(obj_name, validator_cls=None, **validator_kwargs):
             errs.append(error("SS03"))
         if doc.summary != doc.summary.lstrip():
             errs.append(error("SS04"))
-        elif doc.is_function_or_method and doc.summary.split(" ")[0][-1] == "s":
+        # Heuristic to check for infinitive verbs - shouldn't end in "s"
+        elif (
+            doc.is_function_or_method
+            and len(doc.summary.split(" ")[0]) > 1
+            and doc.summary.split(" ")[0][-1] == "s"
+            and doc.summary.split(" ")[0][-2] != "s"
+        ):
             errs.append(error("SS05"))
         if doc.num_summary_lines > 1:
             errs.append(error("SS06"))
 
-    if not doc.extended_summary:
+    if not doc.is_mod and not doc.extended_summary:
         errs.append(("ES01", "No extended summary found"))
 
     # PR01: Parameters not documented
@@ -729,20 +773,21 @@ def validate(obj_name, validator_cls=None, **validator_kwargs):
         if not doc.yields and doc.is_generator_function:
             errs.append(error("YD01"))
 
-    if not doc.see_also:
-        errs.append(error("SA01"))
-    else:
-        for rel_name, rel_desc in doc.see_also.items():
-            if rel_desc:
-                if not rel_desc.endswith("."):
-                    errs.append(error("SA02", reference_name=rel_name))
-                if rel_desc[0].isalpha() and not rel_desc[0].isupper():
-                    errs.append(error("SA03", reference_name=rel_name))
-            else:
-                errs.append(error("SA04", reference_name=rel_name))
+    if not doc.is_mod:
+        if not doc.see_also:
+            errs.append(error("SA01"))
+        else:
+            for rel_name, rel_desc in doc.see_also.items():
+                if rel_desc:
+                    if not rel_desc.endswith("."):
+                        errs.append(error("SA02", reference_name=rel_name))
+                    if rel_desc[0].isalpha() and not rel_desc[0].isupper():
+                        errs.append(error("SA03", reference_name=rel_name))
+                else:
+                    errs.append(error("SA04", reference_name=rel_name))
 
-    if not doc.examples:
-        errs.append(error("EX01"))
+        if not doc.examples:
+            errs.append(error("EX01"))
 
     errs = [err for err in errs if err[0] not in ignore_validation_comments]
 
